//------------------------------------------------------------------
//
//
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  PaleVioletRed
#property indicator_color2  LimeGreen
#property indicator_color3  DarkGray
#property indicator_width3  2

extern string TimeFrame    = "Current time frame";
extern int    period          = 5;
extern bool   alertsOn        = false;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;

double cfhd[];
double cfhu[];
double cfl[];
double trend[];
string indicatorFileName;
bool   returnBars;
int    timeFrame;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,cfhd); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,cfhu); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,cfl);
   SetIndexBuffer(3,trend);
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame == "returnBars"; if (returnBars) return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { cfhd[0] = limit+1; return(0); }
         if (timeFrame != Period())
         {
            limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
            for(int i=limit; i>=0; i--)
            {
               int y = iBarShift(NULL,timeFrame,Time[i]);               
                  cfhd[i] = iCustom(NULL,timeFrame,indicatorFileName,"",period,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,0,y);
                  cfhu[i] = iCustom(NULL,timeFrame,indicatorFileName,"",period,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,1,y);
                  cfl[i]  = iCustom(NULL,timeFrame,indicatorFileName,"",period,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,2,y);
            }
            return(0);
         }

   //
   //
   //
   //
   //

      double N=0,M=0;
   	for (int cnt=period; cnt>=1; cnt--)
      { 
         N += cnt*cnt;
         M += cnt;
      }
      for(i = limit; i >= 0; i--)
      {
         double ZYVar         = 0;
         double TYVar         = 0;
         double ZIndicatorVar = 0;
         double TIndicatorVar = 0;
         for (int k=period; k>0; k--) 
         {
            double tmp = iMA(NULL,0,1,0,MODE_SMA,PRICE_MEDIAN ,i+k-1);
                        ZYVar += tmp*(period-k+1);
                        TYVar += tmp;
                   tmp = iMA(NULL,0,5,0,MODE_SMMA,PRICE_MEDIAN,i+k+2);
                        ZIndicatorVar += tmp*(period-k+1);
                        TIndicatorVar += tmp;
         }
         double AY         = (TYVar        +(N-2*ZYVar        )*period/M)/M;
         double AIndicator = (TIndicatorVar+(N-2*ZIndicatorVar)*period/M)/M;
      	if (StringFind(Symbol(),"JPY",0)>=2)
      	     cfl[i]   = ( 1000)*MathLog(AY/AIndicator);
      	else cfl[i]   = (-1000)*MathLog(AY/AIndicator);
      	     cfhu[i]  = EMPTY_VALUE;
      	     cfhd[i]  = EMPTY_VALUE;
      	     trend[i] = trend[i+1];
      	     if (cfl[i] > 0) { cfhu[i]= cfl[i]; trend[i] =  1; }
      	     if (cfl[i] < 0) { cfhd[i]= cfl[i]; trend[i] = -1; }
   }
   manageAlerts();
   return(0);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      
      //
      //
      //
      //
      //
      
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(" up");
         if (trend[whichBar] == -1) doAlert(" down");
      }
   }
}

//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[0]) {
       previousAlert  = doWhat;
       previousTime   = Time[0];

       //
       //
       //
       //
       //

       message =  StringConcatenate(TimeToStr(TimeLocal(),TIME_SECONDS)," CoefOfLine trend changed to "+doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"CoefOfLine"),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}